from typing import Callable, Dict
from agt import ConversationState
from agt.state import Outputs
from coco.async_api import ComponentSession
from coco.coco import CoCoResponse


async def emit_responses(state: ConversationState, component_response: CoCoResponse):
    if hasattr(component_response, "responses") and component_response.responses:
        for resp in component_response.responses:
            await state.say(
                text=resp.get("text"),
                image_url=resp.get("image"),
                ssml=resp.get("ssml"),
            )
        return
    if component_response.response:
        await state.say(component_response.response)


async def coco(
    state, component_id, user_input=None, context={}, params={}, **extra_params
):
    component_session = ComponentSession(component_id, state.session_id)

    if not user_input:
        user_input = await state.user_input()

    component_response = await component_session(
        user_input,
        context=context,
        parameters={**extra_params, **params},
        flatten_context=True,
        source_language_code=state.memory.get("source_language_code"),
    )
    state.memory = {**state.memory, **component_response.updated_context}

    while not component_response.component_done:
        await emit_responses(state, component_response)

        if component_response.out_of_context:
            await state.out_of_context(state.last_user_input())
        user_input = await state.user_input()
        component_response = await component_session(
            user_input,
            context=context,
            parameters={**extra_params, **params},
            flatten_context=True,
            source_language_code=state.memory.get("source_language_code"),
        )
        state.memory = {**state.memory, **component_response.updated_context}

    await emit_responses(state, component_response)

    return Outputs(
        success=not component_response.component_failed, **component_response.outputs
    )


from .std import oneturn

LOCAL_COMPS: Dict[str, Callable] = {
    "oneturn_say_v2": oneturn.oneturn_say_v2,
    "navigation": oneturn.navigation,
}


async def coco_with_local(state, component_id, *args, **kwargs):
    if component_id in LOCAL_COMPS:
        return await LOCAL_COMPS[component_id](state, *args, **kwargs)
    else:
        return await coco(state, component_id, *args, **kwargs)



import asyncio
import typer
import pathlib
import json
import logging

from typing import Optional

from dotenv import load_dotenv, find_dotenv
from aioconsole import ainput

from .state import ConversationState, Message

load_dotenv(find_dotenv(usecwd=True))

shell_app = typer.Typer(
    name="agt-shell-app", help="agt CLI, test and deploy your projects"
)


async def input_loop(s) -> None:
    while True:
        await s.put_user_input(await ainput())


async def console_output(message: Message, *args, **kwargs):
    print(f"Bot: {message.text}")


async def bot_init(bot, *args, **kwargs) -> None:
    s = ConversationState(console_output)
    s.memory["user_id"] = "shelluser"

    async def wrap_bot(bot, *args, **kwargs):
        try:
            return await bot(*args, **kwargs)
        except Exception as e:
            logging.exception(e)
            raise e

    await asyncio.gather(input_loop(s), wrap_bot(bot, s, *args, **kwargs))


def bot_runner(bot, *args, **kwargs):
    loop = asyncio.get_event_loop()
    loop.run_until_complete(bot_init(bot, *args, **kwargs))


@shell_app.command()
def run(
    component: str = typer.Argument(
        ..., help="example - module_a.module_b:agent_func_name"
    ),
    config: pathlib.Path = typer.Option(
        None, help="A config json file to test with a config"
    ),
):
    """
    Run an Agent COMPONENT in the shell for testing
    """
    import importlib
    import sys

    sys.path.append(".")

    entry_package_name, entry_name = component.split(":")

    entry_package = importlib.import_module(entry_package_name)
    entry = getattr(entry_package, entry_name)

    config_data = None
    if config:
        with open(config, "r") as f:
            config_data = json.load(f)
        bot_runner(entry, config=config_data)
    else:
        bot_runner(entry)


@shell_app.command()
def deploy(
    config: pathlib.Path = typer.Option(
        pathlib.Path("component.yaml"), help="component yaml configuration file path"
    )
):
    """
    Deploy an agent project to CoCoHub cloud according to yaml configuration
    """
    from agt.deploy import deploy as agt_deploy

    asyncio.run(agt_deploy(str(config), app_name=shell_app.info.name or "agt"))


@shell_app.command()
def serve(
    component: str = typer.Argument(
        ..., help="component to serve - format is module:agt_comp_name"
    ),
    component_id: Optional[str] = typer.Option(
        None,
        "--component_id",
        "--component-id",
        help="Component id on cocohub if different from the Agent component name (/api/exchange/<component_id>)",
    ),
    config: Optional[str] = typer.Option(
        None,
        help="Optional config for the component to publish on the hub (/api/config/<component_id>)",
    ),
    port: int = typer.Option(
        8080, "--port", "-p", help="port to serve the component on"
    ),
):
    """
    Serve on a local http server a component with cocohub exchange protocol
    """
    import sys
    import importlib
    from agt.cocohub_vendor import AgentCoCoApp

    sys.path.append(".")

    agent_app = AgentCoCoApp()

    comp_module_path, comp_module_name = component.rsplit(":", maxsplit=1)
    comp_module = importlib.import_module(comp_module_path)
    comp = getattr(comp_module, comp_module_name)

    config_obj = None
    if config:
        config_module_path, config_module_name = config.rsplit(":", maxsplit=1)
        config_module = importlib.import_module(config_module_path)
        config_obj = getattr(config_module, config_module_name)

    agent_app.add_blueprint(comp, component_id=component_id, config=config_obj)

    agent_app.run(host="0.0.0.0", port=port)



import os
import sys
import importlib
import logging

from aiogram import Bot, Dispatcher, executor, types

from agt.server import AgentSessionsManager

API_TOKEN = os.environ["TELEGRAM_TOKEN"]

# Configure logging
logging.basicConfig(filename="agt-telegram.log", level=logging.DEBUG)

# Initialize bot and dispatcher
bot = Bot(token=API_TOKEN)
dp = Dispatcher(bot)


_, entry_path, *args = sys.argv

entry_package_name, entry_name = entry_path.rsplit(".", 1)

entry_package = importlib.import_module(entry_package_name)
entry = getattr(entry_package, entry_name)

sess_mgr = AgentSessionsManager()


class OutputCallback:
    def __init__(self, chat_id) -> None:
        self.chat_id = chat_id

    async def __call__(self, text, image_url=None, *args, **kwargs):
        await bot.send_message(self.chat_id, text)


@dp.message_handler()
async def on_message(message: types.Message):

    session_id = message.chat.id

    output_callback = OutputCallback(message.chat.id)

    sc = sess_mgr.get_session(session_id, entry, output_callback)

    sc.conv_state.memory["user_id"] = message.chat.id

    await sc.conv_state.put_user_input(message.text)


if __name__ == "__main__":
    executor.start_polling(dp, skip_updates=True)



import re


class RegexIntent:
    def __init__(self, *patterns):
        self.patterns = [re.compile(p, re.IGNORECASE) for p in patterns]

    def __call__(self, user_input):
        for p in self.patterns:
            if p.match(user_input):
                return True
        return False


class RegexExtractor:
    def __init__(self, pattern, target_group) -> None:
        self.pattern = re.compile(pattern, re.IGNORECASE)
        self.target_group = target_group

    def __call__(self, user_input):
        m = self.pattern.match(user_input)
        if m:
            return m.group(self.target_group)



import logging
import os
import sys
import importlib

from aiohttp import web
from aiohttp.web import Request, Response, json_response
from botbuilder.core import (
    BotFrameworkAdapterSettings,
    TurnContext,
    BotFrameworkAdapter,
)
from botbuilder.core import ActivityHandler, MessageFactory
from botbuilder.schema import Activity

from ..server import AgentSessionsManager

logging.basicConfig(filename="agt-msbf.log", level=logging.DEBUG)
PORT = os.environ.get("PORT", 3978)
APP_ID = os.environ.get("MicrosoftAppId", "")
APP_PASSWORD = os.environ.get("MicrosoftAppPassword", "")

# Create adapter.
# See https://aka.ms/about-bot-adapter to learn more about how bots work.
SETTINGS = BotFrameworkAdapterSettings(APP_ID, APP_PASSWORD)
ADAPTER = BotFrameworkAdapter(SETTINGS)

sess_mgr = AgentSessionsManager()


class OutputCallback:
    def __init__(self, turn_context) -> None:
        self.turn_context = turn_context

    async def __call__(self, text, image_url=None, *args, **kwargs):
        if image_url:
            msg = MessageFactory.content_url(
                url=image_url, content_type="image/" + image_url[-3:], text=text
            )
        else:
            msg = MessageFactory.text(text)
        await self.turn_context.send_activity(msg)


class PuppetBot(ActivityHandler):
    async def on_message_activity(self, turn_context: TurnContext):

        session_id = turn_context.activity.conversation.id

        output_callback = OutputCallback(turn_context)

        sc = sess_mgr.get_session(session_id, entry, output_callback)

        sc.conv_state.memory["user_id"] = session_id

        await sc.conv_state.put_user_input(turn_context.activity.text)


# Create the Bot
BOT = PuppetBot()


# Listen for incoming requests on /api/messages
async def messages(req: Request) -> Response:
    # Main bot message handler.
    if "application/json" in req.headers["Content-Type"]:
        body = await req.json()
    else:
        return Response(status=415)

    activity = Activity().deserialize(body)
    auth_header = req.headers["Authorization"] if "Authorization" in req.headers else ""

    response = await ADAPTER.process_activity(activity, auth_header, BOT.on_turn)
    if response:
        return json_response(data=response.body, status=response.status)
    return Response(status=201)


APP = web.Application()
APP.router.add_post("/api/messages", messages)

if __name__ == "__main__":
    _, entry_path, *args = sys.argv

    entry_package_name, entry_name = entry_path.rsplit(".", 1)

    entry_package = importlib.import_module(entry_package_name)
    entry = getattr(entry_package, entry_name)

    try:
        web.run_app(APP, host="localhost", port=PORT)
    except Exception as error:
        raise error
